import uuid
import pandas


class DSDataFrame:

    def __init__(self, df_id=None):
        if df_id is None:
            self.df_id = uuid.uuid4()
        else:
            self.df_id = df_id
        self.generatedById = None
        self.dataframe = None

    def from_json(self, json):
        if "id" in json.keys():
            self.df_id = uuid.UUID(json["id"])

        if "generatedById" in json.keys():
            self.generatedById = json["generatedById"]

    def to_json(self):
        obj = {
            "id": str(self.df_id)
        }
        if self.generatedById is not None:
            obj["generatedById"] = str(self.generatedById)
        return obj




import json
import pandas
import time

from .mlmodel import MLModel
from .script import Script
from .plot import Plot
from .wizard_function import WizardStep, WizardFunction
from .ds_dataframe import DSDataFrame


class DSAPIEncoder(json.JSONEncoder):

    def default(self, obj):
        if isinstance(obj, MLModel):
            json_obj = {
                "id": str(obj.model_id),
                "status": str(obj.status),
                "generatedById": str(obj.generatedById),
                "input_columns": list(obj.input_columns),
                "output_columns": list(obj.output_columns),
                "has_anomalies": str(obj.has_anomalies),
                "labels_count": str(obj.label_counts),
                "has_target_feat": str(obj.has_target_feat),
                "needExactColumnNames": str(obj.needExactColumnNames),
                "needExactColumnNumbers": str(obj.needExactColumnNumbers)
            }
            return json_obj
        if isinstance(obj, Script):
            json_obj = {
                "id": str(obj.script_id),
                "name": str(obj.name),
                "description": str(obj.description),
                "canGeneratePlot": str(obj.canGeneratePlot),
                "canGenerateModel": str(obj.canGenerateModel),
                "canGenerateData": str(obj.canGenerateData),
                "status": str(obj.status),
                "needExactColumnNumbers": str(obj.needExactColumnNumbers),
                "needExactColumnNames": str(obj.needExactColumnNames),
                "inputColumns": list(obj.inputColumns),
                "outputColumns": list(obj.outputColumns)
            }
            return json_obj
        if isinstance(obj, DSDataFrame):
            json_obj = {
                "id": str(obj.df_id)
            }
            if obj.generatedById is not None:
                json_obj["generatedById"] = str(obj.generatedById)
            return json_obj
        if isinstance(obj, Plot):
            json_obj = {
                "id": str(obj.plot_id)
            }
            if obj.name is not None:
                json_obj["name"] = obj.name
            if obj.figure is not None:
                json_obj["figure"] = obj.figure
            if obj.experiment_id is not None:
                json_obj["experiment_id"] = obj.experiment_id
            return json_obj
        if isinstance(obj, pandas.Timestamp):
            return int(time.mktime(obj.timetuple()) * 1000)
        if isinstance(obj, WizardStep):
            json_obj = {
                "id": obj.step_id,
                "order": obj.order
            }
            return json_obj
        if isinstance(obj, WizardFunction):
            json_obj = {
                "title": obj.title,
                "function": obj.function,
                "is_beta": str(obj.is_beta),
                "steps": obj.steps
            }
            return json_obj
        else:
            type_name = obj.__class__.__name__
            raise TypeError("Unexpected type {0}".format(type_name))



import uuid
from flask import jsonify


class MLModel:

    def __init__(self, model_id=None):

        # Model ID (GUID used to store the entity on backend and the wizatads on the blob storage
        if model_id is None:
            model_id = uuid.uuid4()
        self.model_id = model_id

        self.generatedById = None
        self.status = 'draft'
        self.needExactColumnNumbers = True
        self.needExactColumnNames = True
        self.input_columns = []
        self.output_columns = []
        self.has_anomalies = False
        self.label_counts = 0
        self.has_target_feat = False

        self.trained_model = None
        self.scaler = None

    def jsonify(self):
        return jsonify({
            "id": str(self.model_id),
            "status": str(self.status),
            "generatedById": str(self.generatedById),
            "input_columns": list(self.input_columns),
            "output_columns": list(self.output_columns),
            "has_anomalies": str(self.has_anomalies),
            "labels_count": str(self.label_counts),
            "has_target_feat": str(self.has_target_feat),
            "needExactColumnNames": str(self.needExactColumnNames),
            "needExactColumnNumbers": str(self.needExactColumnNumbers)
        })

    def load_json(self, json):
        if "id" in json.keys():
            self.model_id = json["id"]
        if "labels_count" in json.keys():
            self.label_counts = int(json["labels_count"])
        if "input_columns" in json.keys():
            self.input_columns = json["input_columns"]
        if "output_columns" in json.keys():
            self.output_columns = json["output_columns"]
        if "has_anomalies" in json.keys():
            self.has_anomalies = json["has_anomalies"]
        if ("has_target_feat" in json.keys()) and (json["has_target_feat"] == 'True'):
            self.has_target_feat = True
        if "needExactColumnNumbers" in json.keys():
            self.needExactColumnNumbers = bool(json["needExactColumnNumbers"] == 'True')
        if "needExactColumnNames" in json.keys():
            self.needExactColumnNames = bool(json["needExactColumnNames"] == 'True')
        if "status" in json.keys():
            self.status = json["status"]

    def get_sample_payload(self):
        pl_columns = {"timestamp": "[timestamp]"}
        for hardwareId in self.input_columns:
            pl_columns[hardwareId] = "[" + hardwareId + "]"
        pl_json = {
            "id": str(self.model_id),
            "dataset": pl_columns
        }
        return pl_json

